// src/from-schema.ts
import {
  any,
  array,
  bigint,
  boolean,
  date,
  func,
  intersection,
  literal,
  map,
  never,
  nullable,
  nullType,
  number,
  object,
  optional,
  record,
  set,
  string,
  tuple,
  undefinedType,
  union,
  unknown,
  UnsupportedError,
  voidType
} from "@onetyped/core";
import { ZodFirstPartyTypeKind } from "zod";
var fromZodSchema = (schema) => {
  const typeName = schema._def.typeName;
  switch (typeName) {
    case ZodFirstPartyTypeKind.ZodString: {
      return string();
    }
    case ZodFirstPartyTypeKind.ZodNumber: {
      return number();
    }
    case ZodFirstPartyTypeKind.ZodBoolean: {
      return boolean();
    }
    case ZodFirstPartyTypeKind.ZodNull: {
      return nullType();
    }
    case ZodFirstPartyTypeKind.ZodUndefined: {
      return undefinedType();
    }
    case ZodFirstPartyTypeKind.ZodUnknown: {
      return unknown();
    }
    case ZodFirstPartyTypeKind.ZodLiteral: {
      return literal(schema._def.value);
    }
    case ZodFirstPartyTypeKind.ZodVoid: {
      return voidType();
    }
    case ZodFirstPartyTypeKind.ZodAny: {
      return any();
    }
    case ZodFirstPartyTypeKind.ZodNever: {
      return never();
    }
    case ZodFirstPartyTypeKind.ZodBigInt: {
      return bigint();
    }
    case ZodFirstPartyTypeKind.ZodDate: {
      return date();
    }
    case ZodFirstPartyTypeKind.ZodObject: {
      const propertySchemas = Object.fromEntries(
        Object.entries(schema._def.shape()).map(([key, value]) => [
          key,
          fromZodSchema(value)
        ])
      );
      return object(propertySchemas);
    }
    case ZodFirstPartyTypeKind.ZodArray: {
      return array(fromZodSchema(schema._def.type));
    }
    case ZodFirstPartyTypeKind.ZodSet: {
      return set(fromZodSchema(schema._def.type));
    }
    case ZodFirstPartyTypeKind.ZodRecord: {
      return record(
        fromZodSchema(schema._def.keyType),
        fromZodSchema(schema._def.valueType)
      );
    }
    case ZodFirstPartyTypeKind.ZodMap: {
      return map(
        fromZodSchema(schema._def.keyType),
        fromZodSchema(schema._def.valueType)
      );
    }
    case ZodFirstPartyTypeKind.ZodFunction: {
      const argumentSchemas = schema._def.args.map(
        (argument) => fromZodSchema(argument)
      );
      const returnSchema = typeof schema._def.returns === "undefined" ? void 0 : fromZodSchema(schema._def.returns);
      return func({ arguments: argumentSchemas, return: returnSchema });
    }
    case ZodFirstPartyTypeKind.ZodUnion: {
      const schemas = schema._def.options.map(
        (option) => fromZodSchema(option)
      );
      return union(schemas);
    }
    case ZodFirstPartyTypeKind.ZodTuple: {
      const schemas = schema._def.items.map(
        (item) => fromZodSchema(item)
      );
      return tuple(schemas);
    }
    case ZodFirstPartyTypeKind.ZodIntersection: {
      const schemas = schema._def.options.map(
        (option) => fromZodSchema(option)
      );
      return intersection(schemas);
    }
    case ZodFirstPartyTypeKind.ZodOptional: {
      return optional(fromZodSchema(schema._def.innerType));
    }
    case ZodFirstPartyTypeKind.ZodNullable: {
      return nullable(fromZodSchema(schema._def.innerType));
    }
    default: {
      throw new UnsupportedError("zod", typeName);
    }
  }
};

// src/to-schema.ts
import {
  mapMultipleInnerNode,
  mapObjectNode
} from "@onetyped/core";
import { z as z2 } from "zod";
var toZodSchema = (node, definitions = /* @__PURE__ */ new Map()) => {
  const zodSchemaDefinitions = /* @__PURE__ */ new Map();
  const schema = toZodSchemaInternal(node, definitions, zodSchemaDefinitions);
  return schema;
};
var toZodSchemaInternal = (node, nodeDefinitions, zodSchemaDefinitions) => {
  switch (node.typeName) {
    case "definitionReference": {
      const schemaBag = zodSchemaDefinitions.get(node.identifier);
      if (schemaBag) {
        return z2.lazy(() => {
          return schemaBag.schema;
        });
      }
      const nodeDefinition = nodeDefinitions.get(node.identifier);
      const resolvedSchemaBag = { schema: z2.unknown() };
      zodSchemaDefinitions.set(node.identifier, resolvedSchemaBag);
      resolvedSchemaBag.schema = toZodSchemaInternal(
        nodeDefinition,
        nodeDefinitions,
        zodSchemaDefinitions
      );
      return z2.lazy(() => resolvedSchemaBag.schema);
    }
    case "string": {
      return z2.string();
    }
    case "number": {
      return z2.number();
    }
    case "boolean": {
      return z2.boolean();
    }
    case "null": {
      return z2.null();
    }
    case "undefined": {
      return z2.undefined();
    }
    case "literal": {
      return z2.literal(node.type);
    }
    case "unknown": {
      return z2.unknown();
    }
    case "void": {
      return z2.void();
    }
    case "any": {
      return z2.any();
    }
    case "never": {
      return z2.never();
    }
    case "regexp": {
      return z2.instanceof(RegExp);
    }
    case "bigint": {
      return z2.bigint();
    }
    case "date": {
      return z2.date();
    }
    case "file": {
      return z2.instanceof(File);
    }
    case "blob": {
      return z2.instanceof(Blob);
    }
    case "formData": {
      return z2.instanceof(FormData);
    }
    case "object": {
      const propertySchemas = Object.fromEntries(
        mapObjectNode(
          node,
          (node2) => toZodSchemaInternal(node2, nodeDefinitions, zodSchemaDefinitions)
        )
      );
      return z2.object(propertySchemas);
    }
    case "array": {
      return z2.array(
        toZodSchemaInternal(node.type, nodeDefinitions, zodSchemaDefinitions)
      );
    }
    case "set": {
      return z2.set(
        toZodSchemaInternal(node.type, nodeDefinitions, zodSchemaDefinitions)
      );
    }
    case "record": {
      return z2.record(
        toZodSchemaInternal(node.key, nodeDefinitions, zodSchemaDefinitions),
        toZodSchemaInternal(node.value, nodeDefinitions, zodSchemaDefinitions)
      );
    }
    case "map": {
      return z2.map(
        toZodSchemaInternal(node.key, nodeDefinitions, zodSchemaDefinitions),
        toZodSchemaInternal(node.value, nodeDefinitions, zodSchemaDefinitions)
      );
    }
    case "function": {
      const argumentSchemas = z2.tuple(
        node.arguments.map(
          (argument) => toZodSchemaInternal(argument, nodeDefinitions, zodSchemaDefinitions)
        )
      );
      if (node.return) {
        return z2.function(
          argumentSchemas,
          toZodSchemaInternal(
            node.return,
            nodeDefinitions,
            zodSchemaDefinitions
          )
        );
      }
      return z2.function(argumentSchemas);
    }
    case "union": {
      if (node.types.length === 1)
        return toZodSchemaInternal(
          node.types[0],
          nodeDefinitions,
          zodSchemaDefinitions
        );
      const schemas = mapMultipleInnerNode(
        node,
        (node2) => toZodSchemaInternal(node2, nodeDefinitions, zodSchemaDefinitions)
      );
      return z2.union(schemas);
    }
    case "tuple": {
      const schemas = mapMultipleInnerNode(
        node,
        (node2) => toZodSchemaInternal(node2, nodeDefinitions, zodSchemaDefinitions)
      );
      return z2.tuple(schemas);
    }
    case "intersection": {
      const types = [...node.types];
      let schema = toZodSchemaInternal(
        types.pop(),
        nodeDefinitions,
        zodSchemaDefinitions
      );
      for (const type of types) {
        schema = schema.and(
          toZodSchemaInternal(type, nodeDefinitions, zodSchemaDefinitions)
        );
      }
      return schema;
    }
    case "named": {
      return toZodSchemaInternal(
        node.node,
        nodeDefinitions,
        zodSchemaDefinitions
      );
    }
    case "iterator": {
      return z2.unknown();
    }
    case "symbol": {
      return z2.unknown();
    }
    case "custom": {
      return z2.unknown();
    }
    case "circular": {
      return z2.unknown();
    }
  }
};

// src/to-string.ts
import {
  mapMultipleInnerNode as mapMultipleInnerNode2,
  mapObjectNode as mapObjectNode2,
  serializePrimitive,
  UnsupportedError as UnsupportedError2
} from "@onetyped/core";
var createZodMethod = (method, argument_) => {
  return `z.${method}(${argument_ ?? ""})`;
};
var toZodString = (node) => {
  switch (node.typeName) {
    case "definitionReference": {
      return createZodMethod("lazy", `() => ${node.identifier}`);
    }
    case "string": {
      return createZodMethod("string");
    }
    case "number": {
      return createZodMethod("number");
    }
    case "boolean": {
      return createZodMethod("boolean");
    }
    case "null": {
      return createZodMethod("null");
    }
    case "undefined": {
      return createZodMethod("undefined");
    }
    case "literal": {
      return createZodMethod("literal", serializePrimitive(node.type));
    }
    case "unknown": {
      return createZodMethod("unknown");
    }
    case "void": {
      return createZodMethod("void");
    }
    case "any": {
      return createZodMethod("any");
    }
    case "never": {
      return createZodMethod("never");
    }
    case "regexp": {
      return createZodMethod("instanceof", "RegExp");
    }
    case "bigint": {
      return createZodMethod("bigint");
    }
    case "date": {
      return createZodMethod("date");
    }
    case "file": {
      return createZodMethod("instanceof", "File");
    }
    case "blob": {
      return createZodMethod("instanceof", "Blob");
    }
    case "formData": {
      return createZodMethod("instanceof", "FormData");
    }
    case "iterator": {
      return createZodMethod("unknown");
    }
    case "symbol": {
      return createZodMethod("unknown");
    }
    case "custom": {
      return createZodMethod("unknown");
    }
    case "object": {
      const propertySchemas = mapObjectNode2(node, toZodString).map(([key, zodType]) => `${JSON.stringify(key)}: ${zodType}`).join(", ");
      return createZodMethod("object", `{ ${propertySchemas} }`);
    }
    case "array": {
      return createZodMethod("array", toZodString(node.type));
    }
    case "set": {
      return createZodMethod("set", toZodString(node.type));
    }
    case "record": {
      return createZodMethod(
        "record",
        `${toZodString(node.key)}, ${toZodString(node.value)}`
      );
    }
    case "map": {
      return createZodMethod(
        "map",
        `${toZodString(node.key)}, ${toZodString(node.value)}`
      );
    }
    case "function": {
      if (node.arguments.length === 0 && typeof node.return === "undefined")
        return createZodMethod("function");
      const argumentSchemas = node.arguments.map(
        (argument) => toZodString(argument)
      );
      const tupleArguments = createZodMethod(
        "tuple",
        `[${argumentSchemas.join(", ")}]`
      );
      if (typeof node.return === "undefined")
        return createZodMethod("function", tupleArguments);
      const returnSchema = toZodString(node.return);
      return createZodMethod("function", `${tupleArguments}, ${returnSchema}`);
    }
    case "union": {
      if (node.types.length === 1)
        return toZodString(node.types[0]);
      const schemas = mapMultipleInnerNode2(node, toZodString).join(", ");
      return createZodMethod("union", `[${schemas}]`);
    }
    case "tuple": {
      const schemas = mapMultipleInnerNode2(node, toZodString).join(", ");
      return createZodMethod("tuple", `[${schemas}]`);
    }
    case "intersection": {
      const types = [...node.types];
      let schema = toZodString(types.pop());
      for (const type of types) {
        schema = `${schema}.and(${toZodString(type)})`;
      }
      return schema;
    }
    case "named": {
      throw new UnsupportedError2("zod", "named");
    }
    case "circular": {
      throw new UnsupportedError2("zod", "circular");
    }
  }
};
export {
  fromZodSchema,
  toZodSchema,
  toZodString
};
